"""
Configs migration

Generation datetime: 2020-12-24 15:58:10.890564
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper as mh, LimitationToModify

REVISION = "6459e7ca"
DOWN_REVISION = "110bed78"

limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="INDEX_PATH",
        oldServices=["luna-indexer"],
        newServices=["luna-indexer", "luna-matcher-daemon"],
        settingUpgradeFunction=None,
        settingDowngradeFunction=None,
    ),
    LimitationToModify(
        limitationName="INDEX_RELOADED_FOLDER",
        oldServices=["luna-indexed-matcher"],
        newServices=["luna-indexed-matcher", "luna-matcher-daemon"],
        settingUpgradeFunction=None,
        settingDowngradeFunction=None,
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="MATCHER_DAEMON",
        validationSchema={
            "type": "object",
            "properties": {
                "indexer_host": {"type": "string"},
                "index_download_dir": {"type": "string"},
                "matcher_service": {"type": "string"},
                "ssh_key": {"type": "string"},
                "store_index_days": {"type": "integer"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
            },
            "required": [
                "indexer_host",
                "index_download_dir",
                "matcher_service",
                "store_index_days",
                "port",
            ],
            "description": "Matcher daemon settings.",
        },
        defaultValue={
            "indexer_host": "127.0.0.1",
            "index_download_dir": "/var/lib/matcher-daemon/index_download",
            "matcher_service": "luna-indexed-matcher",
            "ssh_key": "/etc/matcher-daemon/id_rsa",
            "store_index_days": 2,
            "port": 6001,
        },
        description="Matcher daemon settings.",
        services=["luna-matcher-daemon"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="MATCHER_DAEMON")
