"""
Configs migration

Generation datetime: 2022-04-29 08:53:56.572264
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper, LimitationToModify

mh: DatabaseMigrationHelper

REVISION = "46e61780"
DOWN_REVISION = "822e7d26"


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="INFLUX_MONITORING",
        settingUpgradeFunction=lambda oldValue: oldValue,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
    ),
    LimitationToModify(
        limitationName="STORAGE_TIME",
        settingUpgradeFunction=lambda oldValue: oldValue,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-backport3",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-index-manager",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-backport3",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-index-manager",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
    ),
    LimitationToModify(
        limitationName="LUNA_ACCOUNTS_ADDRESS",
        settingUpgradeFunction=lambda oldValue: oldValue,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-api"],
        newServices=["luna-admin", "luna-api"],
    ),
    LimitationToModify(
        limitationName="LUNA_ACCOUNTS_DB",
        settingUpgradeFunction=lambda oldValue: oldValue,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-admin"],
        newServices=["luna-accounts"],
        oldDescription="Main database for luna-admin service",
        newDescription="Main database for luna-accounts service",
    ),
    LimitationToModify(
        limitationName="STORAGE_TIME",
        settingUpgradeFunction=lambda oldValue: oldValue,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-admin",
            "luna-backport3",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-index-manager",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
        newServices=[
            "luna-accounts",
            "luna-backport3",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-index-manager",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to rename
    mh.renameLimitation(oldName="LUNA_ADMIN_DB", newName="LUNA_ACCOUNTS_DB")

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_ACCOUNTS_ACTIVE_PLUGINS",
        validationSchema={"type": "array", "items": {"type": "string"}},
        defaultValue=[],
        description="list active plugins for luna-accounts",
        services=["luna-accounts"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_ACCOUNTS_LOGGER",
        validationSchema={
            "type": "object",
            "properties": {
                "log_level": {
                    "type": "string",
                    "enum": ["ERROR", "WARNING", "INFO", "DEBUG"],
                },
                "folder_with_logs": {"type": "string"},
                "max_log_file_size": {"type": "integer"},
                "log_time": {"type": "string", "enum": ["LOCAL", "UTC"]},
                "log_to_stdout": {"type": "boolean"},
                "log_to_file": {"type": "boolean"},
                "multiline_stack_trace": {"type": "boolean"},
            },
            "additionalProperties": False,
        },
        defaultValue={
            "log_level": "INFO",
            "folder_with_logs": "./",
            "max_log_file_size": 1024,
            "log_time": "LOCAL",
            "log_to_stdout": True,
            "log_to_file": True,
            "multiline_stack_trace": True,
        },
        description="Luna Accounts logger settings",
        services=["luna-accounts"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_ACCOUNTS_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "origin": {
                    "type": "string",
                    "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
                    "example": "http://127.0.0.1:5051",
                },
                "api_version": {"type": "integer"},
            },
            "required": ["origin", "api_version"],
            "additionalProperties": False,
        },
        defaultValue={"origin": "http://127.0.0.1:5170", "api_version": 1},
        description="LUNA Accounts address",
        services=["luna-api"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_ACCOUNTS_TIMEOUTS",
        validationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        defaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 60,
        },
        description="timeouts for http-request to luna-accounts service",
        services=["luna-api"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_ACCOUNTS_ACTIVE_PLUGINS")
    mh.removeLimitation(limitationName="LUNA_ACCOUNTS_LOGGER")
    mh.removeLimitation(limitationName="LUNA_ACCOUNTS_ADDRESS")
    mh.removeLimitation(limitationName="LUNA_ACCOUNTS_TIMEOUTS")

    # limitations to rename
    mh.renameLimitation(oldName="LUNA_ACCOUNTS_DB", newName="LUNA_ADMIN_DB")
