"""
Configs migration

Generation datetime: 2022-05-18 13:46:46.462161
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper, LimitationToModify

mh: DatabaseMigrationHelper

REVISION = "8fcaa9de"
DOWN_REVISION = "b9d73ee3"


def settingUpgradeFunction(oldSetting):
    if oldSetting["db_type"] == "vertica":
        mh.logger.warning(
            f"The *vertica* database support has been terminated. "
            f"The old `LUNA_EVENTS_DB` setting will be replaced with the new one. The old setting: {oldSetting}"
        )
        return {
            "db_type": "postgres",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_events",
            "db_host": "127.0.0.1",
            "db_port": 5432,
            "db_settings": {"connection_pool_size": 10},
        }
    return oldSetting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_INDEX_MANAGER_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-index-manager"],
        newServices=["luna-index-manager"],
    ),
    LimitationToModify(
        limitationName="LUNA_BACKPORT3_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-backport3"],
        newServices=["luna-backport3"],
    ),
    LimitationToModify(
        limitationName="LUNA_ADMIN_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-admin"],
        newServices=["luna-admin"],
    ),
    LimitationToModify(
        limitationName="LUNA_HANDLERS_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-handlers"],
    ),
    LimitationToModify(
        limitationName="LUNA_FACES_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-faces", "luna-matcher-proxy", "luna-python-matcher"],
        newServices=["luna-faces", "luna-matcher-proxy", "luna-python-matcher"],
    ),
    LimitationToModify(
        limitationName="LUNA_EVENTS_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "db_type": "vertica",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_events",
            "db_host": "127.0.0.1",
            "db_port": 5433,
            "db_settings": {"connection_pool_size": 10},
        },
        newDefaultValue={
            "db_type": "postgres",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_events",
            "db_host": "127.0.0.1",
            "db_port": 5432,
            "db_settings": {"connection_pool_size": 10},
        },
        settingUpgradeFunction=settingUpgradeFunction,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-events", "luna-matcher-proxy", "luna-python-matcher"],
        newServices=["luna-events", "luna-matcher-proxy", "luna-python-matcher"],
    ),
    LimitationToModify(
        limitationName="LUNA_TASKS_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {
                    "type": "string",
                    "enum": ["postgres", "oracle", "vertica"],
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-tasks"],
        newServices=["luna-tasks"],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to remove
    mh.removeLimitation(limitationName="SAVE_EVENTS_INTERVAL")

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="SAVE_EVENTS_INTERVAL",
        validationSchema={"type": "integer"},
        defaultValue=1,
        description="Interval for 'copy' events into db, in seconds",
        services=["luna-events"],
        groupNames=["luna-platform"],
    )
