"""
Configs migration

Generation datetime: 2023-03-29 13:12:52.090805
"""

from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "47a80e3c"
DOWN_REVISION = "2d2d20de"


def upgradeASUDBSetting(setting: dict) -> dict:
    """Upgrade ASU setting."""
    setting["luna_image_store"] = True
    return setting


def downgradeASUDBSetting(setting: dict) -> dict:
    """Downgrade ASU setting."""
    setting.pop("luna_image_store", None)
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="ADDITIONAL_SERVICES_USAGE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "luna_matcher_proxy",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
                "luna_image_store": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "luna_matcher_proxy",
                "luna_image_store",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
        },
        newDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
            "luna_image_store": True,
        },
        oldDescription="whether to use nested optional services",
        newDescription="whether to use nested optional services",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=upgradeASUDBSetting,
        settingDowngradeFunction=downgradeASUDBSetting,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
