"""
Configs migration

Generation datetime: 2025-02-10 13:22:16.952126+00:00
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "e1c45c74"
DOWN_REVISION = "1a4c26dd"


def upgradeSetting(setting: dict) -> dict:
    """Upgrade setting."""
    setting["luna_python_matcher"] = True
    return setting


def downgradeSetting(setting: dict) -> dict:
    """Downgrade setting."""
    setting.pop("luna_python_matcher", None)
    return setting


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="ADDITIONAL_SERVICES_USAGE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_faces": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
                "luna_image_store": {"type": "boolean"},
                "luna_handlers": {"type": "boolean"},
                "luna_lambda": {"type": "boolean"},
                "luna_video_manager": {"type": "boolean"},
                "luna_video_agent": {"type": "boolean"},
                "luna_streams_retranslator": {"type": "boolean"},
            },
            "required": [],
            "description": "whether to use nested optional services",
            "additionalProperties": True,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_faces": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
                "luna_image_store": {"type": "boolean"},
                "luna_handlers": {"type": "boolean"},
                "luna_lambda": {"type": "boolean"},
                "luna_python_matcher": {"type": "boolean"},
                "luna_video_manager": {"type": "boolean"},
                "luna_video_agent": {"type": "boolean"},
                "luna_streams_retranslator": {"type": "boolean"},
            },
            "required": [],
            "description": "whether to use nested optional services",
            "additionalProperties": True,
        },
        oldDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_faces": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
            "luna_image_store": True,
            "luna_handlers": True,
            "luna_lambda": False,
            "luna_video_manager": False,
            "luna_video_agent": False,
            "luna_streams_retranslator": False,
        },
        newDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_faces": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
            "luna_image_store": True,
            "luna_handlers": True,
            "luna_lambda": False,
            "luna_python_matcher": True,
            "luna_video_manager": False,
            "luna_video_agent": False,
            "luna_streams_retranslator": False,
        },
        settingUpgradeFunction=upgradeSetting,
        settingDowngradeFunction=downgradeSetting,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
